[<img src="https://www.bioconductor.org/images/logo/jpg/bioconductor_logo_rgb.jpg" width="200" align="right"/>](https://bioconductor.org/)

**Rsamtools** is an R/Bioconductor package that provides an interface to the `samtools`, `bcftools`, and `tabix` utilities for manipulating SAM (Sequence Alignment / Map), FASTA, binary variant call (BCF) and compressed indexed tab-delimited (tabix) files.

See https://bioconductor.org/packages/Rsamtools for more information including how to install the release version of the package (please refrain from installing directly from GitHub).

