\name{Deprecated and Defunct}
\title{Rsamtools Deprecated and Defunct}

\alias{quickCountBam}
\alias{quickBamCounts}
\alias{bgzipTabix}

\description{

   The function, class, or data object you have asked for has been
   deprecated or made defunct.

}

\keyword{internal}
